import argparse
import copy
import json
from crossref_commons.iteration import iterate_publications_as_json
import xml.etree.ElementTree as ET
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
import pandas as pd
import openpyxl


def parse_input():
    parser = argparse.ArgumentParser(
        description='Get json')
    parser.add_argument(
        '-i', '--input', required=True, help='Input XML file', type=str, nargs="+")
    parser.add_argument(
        '-o', '--output', required=True, help='Output Json file', type=str)
    return parser.parse_args()


def find_doi_using_api(queries):
    for p in iterate_publications_as_json(max_results=1, queries=copy.deepcopy(queries)):
        return p["DOI"], p["score"]


def parse_txt_file_to_read(file_to_read):
    meta_data = {}
    list_of_meta_data = []
    with open(file_to_read, 'r', encoding="utf-8") as reading_file:
        for line in reading_file.readlines():
            split_line = line.split('//')
            if len(split_line) < 3:
                continue
            author, title, suffix = split_line[0], split_line[1], split_line[2]
            meta_data['query.author'] = author
            meta_data['query.title'] = title
            meta_data['query.bibliographic'] = suffix
            list_of_meta_data.append(meta_data)
    return list_of_meta_data


def create_query_using_attributes(meta_data, query):
    if meta_data.tag == "authors":
        authors = []
        for author in meta_data:
            authors.append(author.text)
        query["query.author"] = ", ".join(list(authors))
    if meta_data.tag == "suffix":
        query["query.bibliographic"] = meta_data.text
    if meta_data.tag == "title":
        query["query.title"] = meta_data.text


def create_query_using_text(query):
    query_using_text = {
        "query.bibliographic": ', '.join([query["query.author"], query["query.title"], query["query.bibliographic"]])}
    return query_using_text


def get_doi_from_crossref(query):
    text = ", ".join(
        [query["query.author"], query["query.title"], query["query.bibliographic"]])
    driver = webdriver.Chrome(
        r"C:\ProgramData\chocolatey\lib\chromedriver\tools\chromedriver.exe")
    driver.get("https://search.crossref.org/references")
    text_area = driver.find_element(By.NAME, "references")
    text_area.send_keys(text)
    button = driver.find_element(By.CLASS_NAME, "btn-large")
    button.click()
    wait = WebDriverWait(driver, 5)
    wait.until(EC.presence_of_element_located((By.CLASS_NAME, "page-header")))
    try:
        doi = driver.find_element(
            By.XPATH, "/html/body/div[2]/div[2]/div[2]/table/tbody/tr[2]/td/a").text
    except:
        doi = ""
    driver.quit()
    return doi


def parse_xml(file_to_read):
    output = []
    query = {}
    tree = ET.parse(file_to_read)
    references = tree.getroot()
    for reference in references:
        for meta_data in reference:
            create_query_using_attributes(meta_data, query)
        attribute_meta_data = find_doi_using_api(query)
        text_meta_data = find_doi_using_api(create_query_using_text(query))
        doi_using_attributes, doi_using_attributes_score = "", ""
        doi_using_text, doi_using_text_score = "", ""
        if attribute_meta_data is not None:
            doi_using_attributes, doi_using_attributes_score = attribute_meta_data
        if text_meta_data is not None:
            doi_using_text, doi_using_text_score = text_meta_data
        doi = get_doi_from_crossref(query)
        output.append({
            'query': query,
            'DOI using text': doi_using_text,
            'DOI using text score': doi_using_text_score,
            'DOI using attributes': doi_using_attributes,
            'DOI using attributes score': doi_using_attributes_score,
            'DOI using selenium': doi
        })
        query = {}
    return output


def main() -> None:
    args = parse_input()
    files_to_read = args.input
    file_to_write = args.output

    files_to_read = ["archmath-27-1991-2-8-references.xml", "cmuc-23-1982-3-12-references.xml",
                     "cmuc-31-1990-4-11-references.xml", "kybernetika-50-2014-5-7-references.xml",
                     "kybernetika-55-2019-5-4-references.xml", "mathbohem-134-2009-1-5-references.xml",
                     "mathbohem-143-2018-2-4-references.xml"]

    files_to_read1 = ["archmath-27-1991-2-8-references.xml", "cmuc-23-1982-3-12-references.xml",
                      "cmuc-31-1990-4-11-references.xml", "kybernetika-50-2014-5-7-references.xml",
                      "kybernetika-55-2019-5-4-references.xml", "mathbohem-134-2009-1-5-references.xml",
                      "mathbohem-143-2018-2-4-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-03-1995-1-2-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-06-1998-1-15-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-06-1998-1-20-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-08-2000-1-7-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-09-2001-1-7-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-10-2002-1-12-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-16-2008-1-4-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-17-2009-1-3-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-17-2009-1-5-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-19-2011-2-5-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-20-2012-1-2-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-20-2012-1-7-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-21-2013-2-5-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-22-2014-2-4-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-24-2016-1-5-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-25-2017-2-3-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-26-2018-1-2-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-27-2019-1-4-references.xml",
                      "C:\Users\peter.mercell\data\ActaOstrav-27-2019-2-3-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-030-1994-1-6-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-032-1996-4-9-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-034-1998-1-15-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-034-1998-1-18-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-035-1999-3-2-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-036-2000-3-7-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-036-2000-5-23-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-036-2000-5-28-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-037-2001-4-5-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-040-2004-1-5-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-040-2004-4-3-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-040-2004-4-10-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-045-2009-2-8-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-048-2012-4-4-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-050-2014-3-2-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-051-2015-3-3-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-051-2015-4-1-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-052-2016-1-3-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-052-2016-2-3-references.xml",
                      "C:\Users\peter.mercell\data\ArchMathRetro-053-2017-3-1-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-32-1991-1-8-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-32-1991-2-11-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-32-1991-4-7-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-33-1992-2-6-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-33-1992-3-12-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-34-1993-3-2-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-36-1995-2-11-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-37-1996-3-3-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-39-1998-3-1-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-40-1999-2-14-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-40-1999-2-16-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-41-2000-3-16-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-41-2000-4-12-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-42-2001-1-10-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-43-2002-3-6-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-46-2005-2-6-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-48-2007-4-10-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-52-2011-1-3-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-53-2012-4-5-references.xml",
                      "C:\Users\peter.mercell\data\CommentatMathUnivCarolRetro-54-2013-3-10-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-05-1969-3-3-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-11-1975-6-2-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-13-1977-5-3-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-16-1980-3-4-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-25-1989-5-5-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-28-1992-2-7-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-30-1994-2-2-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-33-1997-5-1-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-34-1998-1-8-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-34-1998-4-20-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-37-2001-4-3-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-37-2001-4-4-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-40-2004-1-9-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-42-2006-5-3-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-42-2006-6-1-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-43-2007-1-8-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-46-2010-2-6-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-46-2010-4-10-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-47-2011-3-5-references.xml",
                      "C:\Users\peter.mercell\data\Kybernetika-53-2017-6-8-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-116-1991-4-1-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-116-1991-4-6-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-120-1995-3-4-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-120-1995-4-7-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-125-2000-2-2-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-125-2000-2-8-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-127-2002-2-6-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-127-2002-2-10-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-127-2002-3-7-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-128-2003-1-5-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-129-2004-4-4-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-130-2005-4-5-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-135-2010-1-5-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-135-2010-4-4-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-138-2013-1-1-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-138-2013-1-2-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-139-2014-2-15-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-139-2014-4-10-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-141-2016-4-1-references.xml",
                      "C:\Users\peter.mercell\data\MathBohem-145-2020-3-7-references.xml"
                      ]
    file_to_write = "output.json"

    output_json = []
    for file_to_read in files_to_read:
        output_json.extend(parse_xml(file_to_read))

    with open(file_to_write, 'w', encoding="utf-8") as output_file:
        json.dump(output_json, output_file)

    df_json = pd.read_json(file_to_write)
    df_json.to_excel("statistics1.xlsx")


if __name__ == '__main__':
    main()
